#include "anime.h"
#include "libmy/lex.h"
#include "libmy/key.h"
#include "img.h"
#include "text.h"
#include "bgm.h"
#include "se.h"
#include "res.h"


//---------------------------------------------------------------------------

ST_ANIME_TABLE AnimePat[ANIME_MAX_PAT_CNT] = {
	{ "image",    (void*)AnimeExecImage   },		// wi\
	{ "image2",   (void*)AnimeExecImage2  },		// Aԕ\
	{ "image3",   (void*)AnimeExecImage3  },		// ^CgS
	{ "effect",   (void*)AnimeExecEffect  },
	{ "update",   (void*)AnimeExecUpdate  },
	{ "text",     (void*)AnimeExecText    },
	{ "text_cls", (void*)AnimeExecTextCls },
	{ "wait",     (void*)AnimeExecWait    },
	{ "key_wait", (void*)AnimeExecKeyWait },
	{ "bgm",      (void*)AnimeExecBgm     },
	{ "bgm_wait", (void*)AnimeExecBgmWait },
	{ "se",       (void*)AnimeExecSe      },
	{ "se_wait",  (void*)AnimeExecSeWait  },
	{ "setval",   (void*)AnimeExecSetVal  },
	{ "setpos",   (void*)AnimeExecSetPos  },
	{ "inc",      (void*)AnimeExecInc     },
	{ "jbe" ,     (void*)AnimeExecJbe     },
	{ "end",      (void*)AnimeExecEnd     },
};

char* AnimeDat[ANIME_MAX_DAT_CNT] = {
	(char*)&ani_1_op_txt,
	(char*)&ani_2_op_txt,
	(char*)&ani_3_op_txt,
	(char*)&ani_4_ed_txt,
};

//---------------------------------------------------------------------------
ST_ANIME Anime;


//---------------------------------------------------------------------------
EWRAM_CODE void AnimeInit(void)
{
	_Memset(&Anime, 0x00, sizeof(ST_ANIME));

	Anime.act = ANIME_ACT_END;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeSetDat(u16 no)
{
	_ASSERT(no < ANIME_MAX_DAT_CNT);


	TRACEOUT("[AnimeSetDat: no:%x]\n", no);
	AnimeSetDat2(AnimeDat[no]);
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeSetDat2(char* pDat)
{
	AnimeInit();
	LexInit();

	LexSetCur(pDat);
	Anime.act = ANIME_ACT_OP;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeSetFunc(void)
{
	char* pStr = LexGetSafeStr();
	TRACEOUT("[AnimeFunc: %s]\n", pStr);


	u16 i;

	for(i=0; i<ANIME_MAX_PAT_CNT; i++)
	{
		if(_Strncmp(pStr, AnimePat[i].pStr, ANIME_MAX_STR_LEN) == 0)
		{
			Anime.pFunc = AnimePat[i].pFunc;
			Anime.step = 0;

			return;
		}
	}

	_ASSERT(0 && "AnimeFunc miss match");
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExec(void)
{
	if(Anime.act == ANIME_ACT_END || ImgIsEnd() == FALSE || TextIsEnd() == FALSE)
	{
		return;
	}

	if(Anime.act == ANIME_ACT_OP)
	{
		AnimeSetFunc();
		Anime.act = ANIME_ACT_FUNC;
	}

	Anime.pFunc();
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecImage(void)
{
	u16 num = LexGetSafeNum();
	ImgSetCg(0, num);

	Anime.act = ANIME_ACT_OP;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecImage2(void)
{
	ImgSetCg(0, Anime.val);

	Anime.act = ANIME_ACT_OP;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecImage3(void)
{
	u16 num = LexGetSafeNum();
	u16 x   = LexGetSafeNum();
	u16 y   = LexGetSafeNum();

	ImgDirect(num, x, y);

	Anime.act = ANIME_ACT_OP;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecEffect(void)
{
	u16 num1 = LexGetSafeNum();
	u16 num2 = LexGetSafeNum();
	ImgSetEffect2(num1, num2);

	Anime.act = ANIME_ACT_OP;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecUpdate(void)
{
	ImgSetDraw();

	Anime.act = ANIME_ACT_OP;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecText(void)
{
	char buf[64] ALIGN(4);

	u16 x = LexGetSafeNum();
	u16 y = LexGetSafeNum();

	char* pStr = LexGetSafeStr();
	AnimeCalcRemoveStrQuart(buf, pStr);

	TextDrawStrXy(x, y, (u16*)buf);
	Anime.act = ANIME_ACT_OP;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecTextCls(void)
{
	TextClearDat();

	Anime.act = ANIME_ACT_OP;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecWait(void)
{
	switch(Anime.step)
	{
	case 0:
		Anime.waitNum = LexGetSafeNum();
		Anime.waitKey = LexGetSafeNum();
		Anime.waitCnt = 0;

		Anime.step++;
		break;

	case 1:
		Anime.waitCnt++;
	
		if((Anime.waitCnt > Anime.waitNum) || (Anime.waitKey == 1 && KeyIsTrgAB() == TRUE))
		{
			Anime.act = ANIME_ACT_OP;
		}
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecKeyWait(void)
{
	if(KeyIsTrgAB() == TRUE)
	{
		Anime.act = ANIME_ACT_OP;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecBgm(void)
{
	u16 num  = LexGetSafeNum();
	u16 num2 = LexGetSafeNum();

	bool isLoop = (num2 == 1) ? TRUE : FALSE;
	BgmPlay(num, isLoop);

	Anime.act = ANIME_ACT_OP;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecBgmWait(void)
{
	if(BgmIsEnd() == TRUE)
	{
		Anime.act = ANIME_ACT_OP;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecSe(void)
{
	u16 num  = LexGetSafeNum();
	u16 num2 = LexGetSafeNum();

	bool isLoop = (num2 == 1) ? TRUE : FALSE;
	SePlay(num, isLoop);

	Anime.act = ANIME_ACT_OP;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecSeWait(void)
{
	if(SeIsEnd() == TRUE)
	{
		Anime.act = ANIME_ACT_OP;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecSetVal(void)
{
	Anime.val = LexGetSafeNum();

	Anime.act = ANIME_ACT_OP;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecSetPos(void)
{
	Anime.pCur = LexGetCur();

	Anime.act = ANIME_ACT_OP;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecInc(void)
{
	Anime.val++;

	Anime.act = ANIME_ACT_OP;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecJbe(void)
{
	u16 num = LexGetSafeNum();

	if(Anime.val <= num)
	{
		LexSetCur(Anime.pCur);
	}

	Anime.act = ANIME_ACT_OP;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecEnd(void)
{
	SeStop();
	BgmStop();

	Anime.act = ANIME_ACT_END;
}
//---------------------------------------------------------------------------
EWRAM_CODE bool AnimeIsEnd(void)
{
	return (Anime.act == ANIME_ACT_END) ? TRUE : FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeCalcRemoveStrQuart(char* pDst, char* pSrc)
{
	// ɓĂ_uNH[g܂

	char* pS = pSrc;
	char* pD = pDst;

	while(*pS != '\0')
	{
		if(*pS == '"')
		{
			pS++;
		}
		else
		{
			*pD++ = *pS++;
		}
	}

	*pD = '\0';
}
