#include "mode3.h"
#include "bios_arm.h"
#include "../res.h"

//---------------------------------------------------------------------------
ST_MODE3 Mode3 EWRAM_BSS;


//---------------------------------------------------------------------------
EWRAM_CODE void Mode3Init(void)
{
	BiosExecFixClear(&Mode3, sizeof(ST_MODE3));

	REG_DISPCNT = MODE_3 | BG2_ON;
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3Exec(void)
{
	if(Mode3.isDraw == TRUE)
	{
		BiosExecFast(Mode3.buf, (void*)VRAM, MODE3_MAX_BUF_SIZE);
		Mode3.isDraw = FALSE;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawBg(u16* pBuf)
{
	BiosExecFast(pBuf, Mode3.buf, MODE3_MAX_BUF_SIZE);
}
//---------------------------------------------------------------------------
// 240x100 p
IWRAM_CODE void Mode3DrawBgSmall(u16* pBuf)
{
	BiosExecFast(pBuf, &Mode3.buf[240 * 30], 240*100*2);
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawBgLz77(u16* pBuf)
{
	LZ77UnCompVram(pBuf, Mode3.buf);
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawBgFill(u16 col)
{
	u32 c =((u32)col << 16) | (u32)col;

	BiosExecFastFix(&c, Mode3.buf, MODE3_MAX_BUF_SIZE);
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawBgClear(void)
{
	Mode3DrawBgFill(RGB8(0, 0, 0));
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawBlend(u16* pImg, u16* pMask, u16 sx, u16 sy, u16 cx, u16 cy)
{
	u16  x, y;
	u16  ex = sx + cx;
	u16  ey = sy + cy;

	for(y=sy; y<ey; y++)
	{
		for(x=sx; x<ex; x++)
		{
			if(*pMask == 0x0000)
			{
				// EMPTY
			}
			else if(*pMask == 0x7fff)
			{
				Mode3.buf[(y * SCREEN_WIDTH) + x] = *pImg;
			}
			else
			{
				u16 a = *pMask & 0x001f;
				u16 s = *pImg;
				u16 d = Mode3.buf[(y * SCREEN_WIDTH) + x];

				u16 sR = (s & 0x001f);
				u16 sG = (s & 0x03e0);
				u16 sB = (s & 0x7c00);

				u16 dR = (d & 0x001f);
				u16 dG = (d & 0x03e0);
				u16 dB = (d & 0x7c00);

				u16 r = ((sR - dR) * a) / 32 + dR;
				u16 g = ((sG - dG) * a) / 32 + dG;
				u16 b = ((sB - dB) * a) / 32 + dB;

				Mode3.buf[(y * SCREEN_WIDTH) + x] = (b & 0x7c00) | (g & 0x03e0) | (r & 0x001f);
			}

			pMask++;
			pImg++;
		}
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawImg(u16 sx, u16 sy, u16 cx, u16 cy, u16* img)
{
	u16* p = Mode3.buf + (sy * SCREEN_WIDTH) + sx;

	u16 x, y;

	for(y=0; y<cy; y++)
	{
		for(x=0; x<cx; x++)
		{
			p[y*SCREEN_WIDTH + x] = img[y*cx + x];
		}
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawImgMask(u16 sx, u16 sy, u16 cx, u16 cy, u16* img, u16 mask)
{
	u16* p = Mode3.buf + (sy * SCREEN_WIDTH) + sx;

	u16 x, y;

	for(y=0; y<cy; y++)
	{
		for(x=0; x<cx; x++)
		{
			if(img[y*cx + x] == mask)
			{
				continue;
			}

			p[y*SCREEN_WIDTH + x] = img[y*cx + x];
		}
	}
}
//---------------------------------------------------------------------------
// ݂͂`FbN
IWRAM_CODE void Mode3DrawImgClip(s16 sx, s16 sy, s16 cx, s16 cy, u16* img)
{
	s16 srcx, srcy;
	s16 dstx, dsty;

	s16 width  = cx;
	s16 height = cy;

	if(sx < 0)
	{
		srcx   = -sx;
		dstx   =   0;
		width +=  sx;
	}
	else
	{
		srcx =  0;
		dstx = sx;
	}

	if(sy < 0)
	{
		srcy    = -sy;
		dsty    =   0;
		height +=  sy;
	}
	else
	{
		srcy =  0;
		dsty = sy;
	}

	if(dstx + width > SCREEN_WIDTH)
	{
		width = SCREEN_WIDTH - dstx;
	}

	if(dsty + height > SCREEN_HEIGHT)
	{
		height = SCREEN_HEIGHT - dsty;
	}

	s16  i, j;
	u16* p;
	u16* q;

	for(i=0; i<height; i++)
	{
		p = img + (srcy + i) * cx + srcx;
		q = Mode3.buf + (dsty + i) * SCREEN_WIDTH + dstx;

		for(j=0; j<width; j++)
		{
//			_ASSERT(q >= Mode3.buf);
//			_ASSERT(q < (Mode3.buf + SCREEN_WIDTH * SCREEN_HEIGHT));
			*q++ |= *p++;
		}
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3Copy(u16 sx, u16 sy, u16 ex, u16 ey)
{
	u16 x, y;

	for(y=sy; y<=ey; y++)
	{
		for(x=sx; x<=ex; x++)
		{
			((u16*)VRAM)[(y * SCREEN_WIDTH) + x] = Mode3.buf[(y * SCREEN_WIDTH) + x];
		}
	}

//	_ASSERT(ex < SCREEN_WIDTH);
//	_ASSERT(ey < SCREEN_HEIGHT);
}
//---------------------------------------------------------------------------
EWRAM_CODE void Mode3SetDraw(bool isDraw)
{
	Mode3.isDraw = isDraw;
}
