#include "menu.h"
#include "libmy/key.h"
#include "libmy/fade.h"
#include "text.h"
#include "siori.h"
#include "nv.h"
#include "history.h"
#include "img.h"
#include "bgm.h"


//---------------------------------------------------------------------------
ST_MENU Menu;


//---------------------------------------------------------------------------
EWRAM_CODE void MenuInit(void)
{
	_Memset(&Menu, 0x00, sizeof(ST_MENU));

	MenuSetAct(MENU_ACT_END);
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuExec(void)
{
	if(Menu.act == MENU_ACT_END)
	{
		return;
	}

	switch(Menu.act)
	{
	case MENU_ACT_READY:
		TextClearDat();
		MenuSetAct(MENU_ACT_DRAW);
		break;

	case MENU_ACT_DRAW:
		MenuExecDraw();
		break;

	case MENU_ACT_SELECT:
		MenuExecSelect();
		break;

	case MENU_ACT_SELECTED:
		MenuExecSelected();
		break;

	case MENU_ACT_END_READY:
		TextDrawRestore();
		MenuSetAct(MENU_ACT_END);
		break;

	default:
		SystemError("[Err] MenuExec Menu.act=%d\n", Menu.act);
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuExecDraw(void)
{
	TextDrawStrXy(Menu.sx, Menu.sy, (u16*)Menu.title);

	switch(Menu.type)
	{
	case MENU_TYPE_NORMAL:
		MenuExecDrawNormal();
		break;

	case MENU_TYPE_STATE:
		MenuExecDrawState();
		break;

	case MENU_TYPE_OPTION:
		MenuExecDrawOption();
		break;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuExecDrawNormal(void)
{
	u16 i;

	for(i=0; i<Menu.regCnt; i++)
	{
		if(i == Menu.sel)
		{
			TextDrawStrXy(Menu.sx, Menu.sy + 2 + i, (u16*)Menu.item[i].str);
		}
		else
		{
			TextDrawStrXyGray(Menu.sx, Menu.sy + 2 + i, (u16*)Menu.item[i].str);
		}
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuExecDrawState(void)
{
	MenuExecDrawNormal();

	typedef struct {
		char* str[3] ALIGN(4);
	} ST_MSG;

	static const ST_MSG Msg[5] = {
		{ {"Ȃ", "邠", ""    }, },
		{ {"OE@", "с@@", ""    }, },
		{ {"OE@", "с@@", ""    }, },
		{ {"OE@", "с@@", "S"}, },
		{ {"OE@", "с@@", "S"}, },
	};

	u16 i, j;

	for(i=0; i<Menu.regCnt; i++)
	{
		for(j=0; j<3; j++)
		{
			if(j == Menu.item[i].cnt)
			{
				TextDrawStrXy(Menu.sx + 4 + j*4, Menu.sy + 2 + i, (u16*)Msg[i].str[j]);
			}
			else
			{
				TextDrawStrXyGray(Menu.sx + 4 + j*4, Menu.sy + 2 + i, (u16*)Msg[i].str[j]);
			}
		}
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuExecDrawOption(void)
{
	MenuExecDrawNormal();

	u16 i;

	for(i=0; i<Menu.regCnt; i++)
	{
		u16  cnt = Menu.item[i].cnt;

		// 0x82 0x4f = SJISR[huOv
		char str[5] ALIGN(4);
		str[0] = 0x82;
		str[1] = (cnt >= 10) ? 0x50 : 0x4f;
		str[2] = 0x82;
		str[3] = (cnt >= 10) ? 0x4f + (cnt - 10) : 0x4f + cnt;
		str[4] = '\0';

		if(i == Menu.sel)
		{
			TextDrawStrXy(Menu.sx + 7, Menu.sy + 2 + i, (u16*)str);
		}
		else
		{
			TextDrawStrXyGray(Menu.sx + 7, Menu.sy + 2 + i, (u16*)str);
		}
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuExecSelect(void)
{
	ST_MENU* p = &Menu;

	p->trg = KeyGetTrg();

	if((p->trg & KEY_B) && (p->isCancel == TRUE))
	{
		if(p->ret == MENU_RET_TITLE)
		{
			MenuSetTitle();
		}
		else if(p->ret == MENU_RET_SYSTEM)
		{
			MenuSetSystem();
		}
		else
		{
			MenuSetAct(MENU_ACT_END_READY);
		}
	}
	else if((p->trg & KEY_UP) && p->sel > 0)
	{
		p->sel--;
		MenuSetAct(MENU_ACT_DRAW);
	}
	else if((p->trg & KEY_DOWN) && (p->sel + 1) < p->regCnt)
	{
		p->sel++;
		MenuSetAct(MENU_ACT_DRAW);
	}
	else if((p->trg & KEY_LEFT) || (p->trg & KEY_RIGHT) || (p->trg & KEY_A))
	{
		if(p->sel != -1)
		{
			MenuSetAct(MENU_ACT_SELECTED);
		}
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuExecSelected(void)
{
	Menu.item[Menu.sel].pFunc();
}
//---------------------------------------------------------------------------
EWRAM_CODE bool MenuIsEnd(void)
{
	return (Menu.act == MENU_ACT_END) ? TRUE : FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuSetAct(u16 act)
{
	Menu.act = act;
	Menu.actStep = 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuSetOption(u16 sx, u16 sy, char* pTitle, u16 type, bool isCancel)
{
	Menu.sx = sx;
	Menu.sy = sy;
	Menu.sel = 0;
	Menu.type = type;
	Menu.regCnt = 0;
	Menu.isCancel = isCancel;
	_Strncpy(Menu.title, pTitle, MENU_MAX_STR_LEN);

	MenuSetAct(MENU_ACT_READY);
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuAddItem(char* pStr, void* pFunc)
{
	_ASSERT(Menu.regCnt <  MENU_MAX_ITEM_CNT);


	_Strncpy(Menu.item[Menu.regCnt].str, pStr, MENU_MAX_STR_LEN);
	Menu.item[Menu.regCnt].pFunc = pFunc;
	Menu.item[Menu.regCnt].cnt = 0;

	Menu.regCnt++;
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuSetTitle(void)
{
	MenuInit();
	MenuSetOption(0, 0, "AgNiNA@uPDOT", MENU_TYPE_NORMAL, FALSE);

	MenuAddItem("͑I",   &MenuItem0Chapter);
	MenuAddItem("ԕύX", &MenuItem0State);
	MenuAddItem("[h",   &MenuItem0Load);
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuSetChapter(void)
{
	MenuInit();
	MenuSetOption(3, 0, "@@|@͑I@|", MENU_TYPE_NORMAL, TRUE);

	MenuAddItem("v[O", &MenuItem1Chapter);
	MenuAddItem("T`z",     &MenuItem1Chapter);
	MenuAddItem("cO~",     &MenuItem1Chapter);
	MenuAddItem("^Jq",   &MenuItem1Chapter);
	MenuAddItem("JiR",     &MenuItem1Chapter);
	MenuAddItem("zRr",   &MenuItem1Chapter);
	MenuAddItem("I",       &MenuItem1Chapter);
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuSetState(void)
{
	MenuInit();
	MenuSetOption(1, 0, "@@@|@ԕύX@|", MENU_TYPE_STATE, TRUE);

	MenuAddItem("Ȃ", &MenuItem2Confirm);
	MenuAddItem("", &MenuItem2Confirm);
	MenuAddItem("", &MenuItem2Confirm);
	MenuAddItem("O@", &MenuItem2Confirm);
	MenuAddItem("av@", &MenuItem2Confirm);

	u16 i;
	for(i=0; i<5; i++)
	{
		Menu.item[i].cnt = NvGetChrFlag(i);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuSetSystem(void)
{
	MenuInit();
	MenuSetOption(3, 0, "|@VXej[@|", MENU_TYPE_NORMAL, TRUE);

	MenuAddItem("",     &MenuItem3Next);
	MenuAddItem("",   &MenuItem3TextHide);
	MenuAddItem("ViIz", &MenuItem3Ref);
	MenuAddItem("Z[u",       &MenuItem3Save);
	MenuAddItem("[h",       &MenuItem3Load);
	MenuAddItem("ԕύX",     &MenuItem3Change);
	MenuAddItem("IvV",   &MenuItem3Option);
	MenuAddItem("Q[I",   &MenuItem3GameEnd);
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuSetSystemOption(void)
{
	MenuInit();
	MenuSetOption(3, 0, "|@VXeݒ@|", MENU_TYPE_OPTION, TRUE);

	MenuAddItem("EFCg", &MenuItem5TextWait);
	MenuAddItem("wiPx@@", &MenuItem5Fade);
	MenuAddItem("afl@@@", &MenuItem5Bgm);
	MenuAddItem("bf@@@@", &MenuItem5Cg);

	Menu.item[0].cnt = TextGetWaitNum();
	Menu.item[1].cnt = FadeGetMsgFade();
	Menu.item[2].cnt = 2;
	Menu.item[3].cnt = 100;
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuSetSave(void)
{
	MenuInit();
	MenuSetOption(3, 0, "@@|@Z[u@|", MENU_TYPE_NORMAL, TRUE);

	u16 i;

	for(i=0; i<SIORI_MAX_CNT; i++)
	{
		MenuAddItem(SioriGetName(i), &MenuItem4SaveSiori);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuSetLoad(void)
{
	MenuInit();
	MenuSetOption(3, 0, "@@|@[h@|", MENU_TYPE_NORMAL, TRUE);

	u16 i;

	for(i=0; i<SIORI_MAX_CNT; i++)
	{
		MenuAddItem(SioriGetName(i), &MenuItem4LoadSiori);
	}
}
//---------------------------------------------------------------------------
// ͑I
EWRAM_CODE void MenuItem0Chapter(void)
{
	if(Menu.trg & KEY_A)
	{
		MenuSetChapter();
		Menu.ret = MENU_RET_TITLE;

		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// ԕύX
EWRAM_CODE void MenuItem0State(void)
{
	if(Menu.trg & KEY_A)
	{
		MenuSetState();
		Menu.ret = MENU_RET_TITLE;

		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// [h
EWRAM_CODE void MenuItem0Load(void)
{
	if(Menu.trg & KEY_A)
	{
		MenuSetLoad();
		Menu.ret = MENU_RET_TITLE;

		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// ͂I
EWRAM_CODE void MenuItem1Chapter(void)
{
	if(Menu.trg & KEY_A)
	{
		NvSetCapter(Menu.sel);
		MenuSetAct(MENU_ACT_END_READY);

		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// ԂI
EWRAM_CODE void MenuItem2Confirm(void)
{
	if(Menu.trg & KEY_LEFT)
	{
		if(Menu.item[Menu.sel].cnt == 0)
		{
			goto End;
		}

		Menu.item[Menu.sel].cnt--;
		NvSetChrFlag(Menu.sel, Menu.item[Menu.sel].cnt);

		MenuSetAct(MENU_ACT_DRAW);
		return;
	}

	if(Menu.trg & KEY_RIGHT)
	{
		u16 sel = Menu.sel;
		u16 cnt = Menu.item[Menu.sel].cnt;

		if(sel <= 2 && cnt == 1)
		{
			goto End;
		}
		else if(cnt == 2)
		{
			goto End;
		}

		Menu.item[Menu.sel].cnt++;
		NvSetChrFlag(Menu.sel, Menu.item[Menu.sel].cnt);

		MenuSetAct(MENU_ACT_DRAW);
		return;
	}

End:
	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// 
EWRAM_CODE void MenuItem3Next(void)
{
	if(Menu.trg & KEY_A)
	{
		NvSetSkip(TRUE);
		MenuSetAct(MENU_ACT_END_READY);

		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// 
EWRAM_CODE void MenuItem3TextHide(void)
{
	switch(Menu.actStep)
	{
	case 0:
		if(Menu.trg & KEY_A)
		{
			Menu.actStep++;
		}
		else
		{
			MenuSetAct(MENU_ACT_SELECT);
		}
		break;

	case 1:
		TextClearDat();
		Menu.actStep++;
		break;

	case 2:
		Menu.trg = KeyGetTrg();

		if(Menu.trg & KEY_A)
		{
			MenuSetAct(MENU_ACT_DRAW);
		}
	}
}
//---------------------------------------------------------------------------
// ViIz
EWRAM_CODE void MenuItem3Ref(void)
{
	switch(Menu.actStep)
	{
	case 0:
		if(Menu.trg & KEY_A && HistoryIsEmpty() == FALSE)
		{
			Menu.actStep++;
		}
		else
		{
			MenuSetAct(MENU_ACT_SELECT);
		}
		break;

	case 1:
		HistoryInitAct(FALSE);
		Menu.actStep++;
		break;

	case 2:
		HistoryExec();

		if(HistoryIsEnd() == TRUE)
		{
			MenuSetAct(MENU_ACT_READY);
		}
		break;
	}
}
//---------------------------------------------------------------------------
// Z[u@VXej[
EWRAM_CODE void MenuItem3Save(void)
{
	if(Menu.trg & KEY_A)
	{
		MenuSetSave();
		Menu.ret = MENU_RET_SYSTEM;

		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// [h@VXej[
EWRAM_CODE void MenuItem3Load(void)
{
	if(Menu.trg & KEY_A)
	{
		MenuSetLoad();
		Menu.ret = MENU_RET_SYSTEM;

		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// ԕύX
EWRAM_CODE void MenuItem3Change(void)
{
	if(Menu.trg & KEY_A)
	{
		MenuSetState();
		Menu.ret = MENU_RET_SYSTEM;

		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// IvV
EWRAM_CODE void MenuItem3Option(void)
{
	if(Menu.trg & KEY_A)
	{
		MenuSetSystemOption();
		Menu.ret = MENU_RET_SYSTEM;

		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// Q[I
EWRAM_CODE void MenuItem3GameEnd(void)
{
	if(Menu.trg & KEY_A)
	{
		HistoryInit();
		TextInit();

		NvSetAct(NV_ACT_END_READY);
		MenuSetAct(MENU_ACT_END_READY);
		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// Z[u@PIς
EWRAM_CODE void MenuItem4SaveSiori(void)
{
	if(Menu.trg & KEY_A)
	{
		SioriSave(Menu.sel);
		MenuSetAct(MENU_ACT_END_READY);

		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// [h@PIς
EWRAM_CODE void MenuItem4LoadSiori(void)
{
	if(Menu.trg & KEY_A)
	{
		if(SioriLoad(Menu.sel) == TRUE)
		{
			MenuSetAct(MENU_ACT_END_READY);
			return;
		}
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// EFCg
EWRAM_CODE void MenuItem5TextWait(void)
{
	if(Menu.trg & KEY_LEFT && Menu.item[0].cnt > 0)
	{
		Menu.item[0].cnt--;
		TextSetWaitNum(Menu.item[0].cnt);

		MenuSetAct(MENU_ACT_DRAW);
		return;
	}
	else if(Menu.trg & KEY_RIGHT && Menu.item[0].cnt < 15)
	{
		Menu.item[0].cnt++;
		TextSetWaitNum(Menu.item[0].cnt);

		MenuSetAct(MENU_ACT_DRAW);
		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// wiPx
EWRAM_CODE void MenuItem5Fade(void)
{
	if(Menu.trg & KEY_LEFT && Menu.item[1].cnt > 0)
	{
		Menu.item[1].cnt--;
		FadeDirect(Menu.item[1].cnt);

		MenuSetAct(MENU_ACT_DRAW);
		return;
	}
	else if(Menu.trg & KEY_RIGHT && Menu.item[1].cnt < 15)
	{
		Menu.item[1].cnt++;
		FadeDirect(Menu.item[1].cnt);

		MenuSetAct(MENU_ACT_DRAW);
		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// afl
EWRAM_CODE void MenuItem5Bgm(void)
{
	if(Menu.trg & KEY_LEFT && Menu.item[2].cnt > 2)
	{
		Menu.item[2].cnt--;
		MenuSetAct(MENU_ACT_DRAW);
		return;
	}
	else if(Menu.trg & KEY_RIGHT && Menu.item[2].cnt < 16)
	{
		Menu.item[2].cnt++;
		MenuSetAct(MENU_ACT_DRAW);
		return;
	}
	else if(Menu.trg & KEY_A)
	{
		BgmDirect(Menu.item[2].cnt, TRUE);
		MenuSetAct(MENU_ACT_DRAW);
		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// bf
EWRAM_CODE void MenuItem5Cg(void)
{
	if(Menu.trg & KEY_LEFT && Menu.item[3].cnt > 100)
	{
		Menu.item[3].cnt--;
		MenuSetAct(MENU_ACT_DRAW);
		return;
	}
	else if(Menu.trg & KEY_RIGHT && Menu.item[3].cnt < 159)
	{
		Menu.item[3].cnt++;
		MenuSetAct(MENU_ACT_DRAW);
		return;
	}
	else if(Menu.trg & KEY_A)
	{
		ImgDirect2(Menu.item[3].cnt);
		MenuSetAct(MENU_ACT_DRAW);
		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
