#ifndef __MENU_H__
#define __MENU_H__
#ifdef __cplusplus
extern "C" {
#endif


#include "libgba/gba.h"

//---------------------------------------------------------------------------
#define MENU_MAX_STR_LEN					36
#define MENU_MAX_ITEM_CNT					8


enum {
	MENU_ACT_READY = 0x01,
	MENU_ACT_DRAW,
	MENU_ACT_SELECT,
	MENU_ACT_SELECTED,
	MENU_ACT_END_READY,
	MENU_ACT_END,
};

enum {
	MENU_TYPE_NORMAL = 0x01,
	MENU_TYPE_STATE,
	MENU_TYPE_OPTION,
};

enum {
	MENU_RET_TITLE = 0x01,
	MENU_RET_SYSTEM,
};

//---------------------------------------------------------------------------

typedef struct {
	u16  cnt;
	char str[MENU_MAX_STR_LEN] ALIGN(4);
	void (*pFunc)(void);
} ST_MENU_ITEM;


typedef struct {
	u16  act;
	u16  actStep;
	u16  sx;										// zuW
	u16  sy;

	u16  type;										// j[̎
	s16  sel;										// I𒆂̏ꏊ
	u16  ret;										// \Kw̑ޔp
	s16  regCnt;									// I̓o^
	bool isCancel;									// LZ̗L
	char title[MENU_MAX_STR_LEN] ALIGN(4);			// ^Cg
	ST_MENU_ITEM item[MENU_MAX_ITEM_CNT] ALIGN(4);	// ACe

	u16  trg;
} ST_MENU;


//---------------------------------------------------------------------------
EWRAM_CODE void MenuInit(void);
EWRAM_CODE void MenuExec(void);
EWRAM_CODE void MenuExecDraw(void);
EWRAM_CODE void MenuExecDrawNormal(void);
EWRAM_CODE void MenuExecDrawState(void);
EWRAM_CODE void MenuExecDrawOption(void);
EWRAM_CODE void MenuExecSelect(void);
EWRAM_CODE void MenuExecSelected(void);
EWRAM_CODE bool MenuIsEnd(void);

EWRAM_CODE void MenuSetAct(u16 act);
EWRAM_CODE void MenuSetOption(u16 sx, u16 sy, char* pTitle, u16 type, bool isCancel);
EWRAM_CODE void MenuAddItem(char* pStr, void* pFunc);

EWRAM_CODE void MenuSetTitle(void);
EWRAM_CODE void MenuSetChapter(void);
EWRAM_CODE void MenuSetState(void);
EWRAM_CODE void MenuSetSystem(void);
EWRAM_CODE void MenuSetSystemOption(void);
EWRAM_CODE void MenuSetSave(void);
EWRAM_CODE void MenuSetLoad(void);


EWRAM_CODE void MenuItem0Chapter(void);
EWRAM_CODE void MenuItem0State(void);
EWRAM_CODE void MenuItem0Load(void);
EWRAM_CODE void MenuItem1Chapter(void);
EWRAM_CODE void MenuItem2Confirm(void);
EWRAM_CODE void MenuItem3Next(void);
EWRAM_CODE void MenuItem3TextHide(void);
EWRAM_CODE void MenuItem3Ref(void);
EWRAM_CODE void MenuItem3Save(void);
EWRAM_CODE void MenuItem3Load(void);
EWRAM_CODE void MenuItem3Change(void);
EWRAM_CODE void MenuItem3Option(void);
EWRAM_CODE void MenuItem3GameEnd(void);
EWRAM_CODE void MenuItem4SaveSiori(void);
EWRAM_CODE void MenuItem4LoadSiori(void);
EWRAM_CODE void MenuItem5TextWait(void);
EWRAM_CODE void MenuItem5Fade(void);
EWRAM_CODE void MenuItem5Bgm(void);
EWRAM_CODE void MenuItem5Cg(void);


#ifdef __cplusplus
}
#endif
#endif
