#include "nv.h"
#include "nv2.h"
#include "libmy/key.h"
#include "file.h"
#include "history.h"
#include "img.h"
#include "anime.h"
#include "menu.h"
#include "bgm.h"
#include "se.h"


//---------------------------------------------------------------------------
// LN^ԁiNv.varj
// 28  Ȃ  0:Ȃ  1:邠
// 32    2:OE    1:
// 36    2:OE    1:
// 40  O    2:OE    1:    3:S
// 44  av    2:OE    1:    3:S

const u16 NvChrFlagTable[NV_MAX_CHR_TABLE_CNT] = {
	28, 32, 36, 40, 44,
};


//---------------------------------------------------------------------------
ST_NV Nv;


//---------------------------------------------------------------------------
EWRAM_CODE void NvInit(void)
{
	_Memset(&Nv, 0x00, sizeof(ST_NV));

	Nv.var[28] = 0;
	Nv.var[32] = 2;
	Nv.var[36] = 2;
	Nv.var[40] = 2;
	Nv.var[44] = 2;
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvInitExceptChr(void)
{
	u16 tbl[NV_MAX_CHR_TABLE_CNT];
	u16 i;

	for(i=0; i<NV_MAX_CHR_TABLE_CNT; i++)
	{
		tbl[i] = Nv.var[ NvChrFlagTable[i] ];
	}

	NvInit();

	for(i=0; i<NV_MAX_CHR_TABLE_CNT; i++)
	{
		Nv.var[ NvChrFlagTable[i] ] = tbl[i];
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExec(void)
{
	if(ImgIsEnd() == FALSE || TextIsEnd() == FALSE || AnimeIsEnd() == FALSE
	   || HistoryIsEnd() == FALSE || MenuIsEnd() == FALSE)
	{
		return;
	}

	if(Nv.act == NV_ACT_END)
	{
		return;
	}


	switch(Nv.act)
	{
	case NV_ACT_PARSE:
		NvExecParse();
		break;

	case NV_ACT_KEY:
		NvExecKey();
		break;

	case NV_ACT_SELECT:
		NvExecSelect();
		break;

	case NV_ACT_END_READY:
		NvExecEndReady();
		break;

	default:
		SystemError("[Err] NvExec Nv.act = %d\n", Nv.act);
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecKey(void)
{
	if(NvExecKeySub() == TRUE)
	{
		NvSetAct(NV_ACT_PARSE);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE bool NvExecKeySub(void)
{
	u16 cnt = KeyGetCnt();
	u16 trg = KeyGetTrg();

	TextShowCur();

	if(Nv.isSkip == TRUE)
	{
		if(cnt & KEY_B)
		{
			Nv.isSkip = FALSE;
		}
		else
		{
			TextHideCur();
			return TRUE;
		}
	}

	if((cnt & KEY_R) || (trg & KEY_A) || (trg & KEY_DOWN))
	{
		TextHideCur();
		return TRUE;
	}
	else if((trg & KEY_LEFT) && HistoryIsEmpty() == FALSE)
	{
		TextHideCur();
		HistoryInitAct(TRUE);

		return FALSE;
	}
	else if(trg & KEY_B)
	{
		TextHideCur();
		MenuSetSystem();

		return FALSE;
	}

	return FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecSelect(void)
{
	u16 i;

	switch(Nv.actStep)
	{
	case 0:
		TextClear();
		Nv.actStep++;
		break;

	case 1:
		Nv.sel = -1;
		Nv.actStep++;
		break;

	case 2:
		for(i=0; i<Nv.selCnt; i++)
		{
			if(Nv.sel == i)
			{
				TextDrawStrXy(0, i, Nv.selBuf[i]);
			}
			else
			{
				TextDrawStrXyGray(0, i, Nv.selBuf[i]);
			}
		}
		Nv.actStep++;
		break;

	case 3: ;
		u16 trg = KeyGetTrg();

		if((trg & KEY_UP) && Nv.sel > 0)
		{
			Nv.sel--;
			Nv.actStep--;
		}
		else if((trg & KEY_DOWN) && (Nv.sel + 1) < Nv.selCnt)
		{
			Nv.sel++;
			Nv.actStep--;
		}
		else if((trg & KEY_A) && Nv.sel != -1)
		{
			Nv.actStep++;
		}
		else if((trg & KEY_LEFT) && HistoryIsEmpty() == FALSE)
		{
			HistoryInitAct(TRUE);
			Nv.actStep--;
		}
		else if(trg & KEY_B)
		{
			MenuSetSystem();
			Nv.actStep--;
		}
		break;

	case 4:
		Nv.var[0] = Nv.selNum[Nv.sel] + 1;
		Nv.selRegCnt = 0;
		Nv.selCnt = 0;
		Nv.isSkip = FALSE;

		NvSetAct(NV_ACT_PARSE);
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecEndReady(void)
{
	BgmStop();
	SeStop();

	ImgSetCg(0, 0);
	ImgSetCg(1, 0);
	ImgSetEffect2(IMG_EFFECT_NORMAL, 0);
	ImgSetDraw();

	TextClear();
	TextShow();

	HistoryInit();

	NvSetAct(NV_ACT_END);
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvSetAct(u16 act)
{
	Nv.act = act;
	Nv.actStep = 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvSetSco(u16 no, u16 adr)
{
	ST_NV_SCO_HEADER* p = (ST_NV_SCO_HEADER*)FileGetSco(no);

	Nv.datAdr = p->datAdr;
	Nv.size   = p->size;
	Nv.no     = no;
	Nv.pStr   = FileGetScoStr(no);
	Nv.pHead  = (u8*)p;

	if(adr == 0x0000)
	{
		Nv.pCur = (u8*)p + p->datAdr;
	}
	else
	{
		Nv.pCur = (u8*)p + adr;
	}

//	TRACEOUT("[Load: str:%s pCur:%x]\n", Nv.pStr, Nv.pCur - Nv.pHead);
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvSetSkip(bool is)
{
	Nv.isSkip = is;
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvSetCapter(u16 no)
{
	NvInitExceptChr();

	switch(no)
	{
	// v[O
	case 0:
		NvCallPush2(90, 2);
		NvCallPush2(22, 0x17d5);
		NvCallPush2(22, 0x1601);
		NvCallPush2(22, 0x1552);
		NvCallPush2(90, 1);
		NvCallPush2(22, 0x066f);
		NvCallPush2(22, 0x0542);
		NvSetSco(22, 0x02d7);
		AnimeSetDat(0);
		break;

	// T`z
	case 1:
		NvCallPush2(27, 0x002a);
		NvCallPush2(26, 0x002a);
		NvCallPush2(25, 0x002a);
		NvCallPush2(24, 0x002a);
		NvSetSco(23, 0x002a);
		break;

	// cO~
	case 2:
		NvCallPush2(31, 0x002a);
		NvCallPush2(30, 0x002a);
		NvCallPush2(29, 0);
		NvSetSco(28, 0);
		break;

	// ^Jq
	case 3:
		// ہ݂OȄꍇAۂтɋύX܂
		// PCł͂ǂ炩тł͂ȂƃX^[gȂł
		if(Nv.var[32] == 2 && Nv.var[36] == 2)
		{
			Nv.var[32] = 1;
		}
		NvSetSco(33, 0);
		break;

	// JiR
	case 4:
		NvCallPush2(43, 0);
		NvCallPush2(42, 0x002a);
		NvCallPush2(41, 0x002a);
		NvCallPush2(40, 0x002a);
		NvSetSco(39, 0x002a);
		break;

	// zRr
	case 5:
		NvCallPush2(52, 0x002a);
		NvCallPush2(51, 0x002a);
		NvCallPush2(50, 0x002a);
		NvCallPush2(49, 0x002a);
		NvCallPush2(48, 0x002a);
		NvSetSco(47, 0);
		break;

	// I
	case 6:
		NvCallPush2(90, 3);
		NvCallPush2(54, 0x002a);
		NvSetSco(53, 0x0029);
		break;
	}

	NvSetAct(NV_ACT_PARSE);
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvSetChrFlag(u16 num, u16 no)
{
	_ASSERT(num < NV_MAX_CHR_TABLE_CNT);


	if(num == 0)
	{
		Nv.var[ NvChrFlagTable[num] ] = no;
		return;
	}

	const u16 tbl[] = { 2, 1, 3 };
	Nv.var[ NvChrFlagTable[num] ] = tbl[no];
}
//---------------------------------------------------------------------------
EWRAM_CODE u16 NvGetChrFlag(u16 num)
{
	_ASSERT(num < NV_MAX_CHR_TABLE_CNT);


	if(num == 0)
	{
		return Nv.var[ NvChrFlagTable[num] ];
	}

	const u16 tbl[] = { 0, 1, 0, 2 };
	return tbl[ Nv.var[ NvChrFlagTable[num] ] ];
}
//---------------------------------------------------------------------------
EWRAM_CODE bool NvIsEnd(void)
{
	return (Nv.act == NV_ACT_END) ? TRUE : FALSE;
}
