#include "nv2.h"
#include "nv.h"
#include "anime.h"
#include "img.h"
#include "text.h"
#include "bgm.h"
#include "se.h"

//---------------------------------------------------------------------------

u16 NvKanaToSjisTable[] = {
	0x8140, 0x8142, 0x8175, 0x8176,
	0x8141, 0x8145, 0x82f0, 0x829f,
	0x82a1, 0x82a3, 0x82a5, 0x82a7,
	0x82e1, 0x82e3, 0x82e5, 0x82c1,
	0x815b, 0x82a0, 0x82a2, 0x82a4,
	0x82a6, 0x82a8, 0x82a9, 0x82ab,
	0x82ad, 0x82af, 0x82b1, 0x82b3,
	0x82b5, 0x82b7, 0x82b9, 0x82bb,
	0x82bd, 0x82bf, 0x82c2, 0x82c4,
	0x82c6, 0x82c8, 0x82c9, 0x82ca,
	0x82cb, 0x82cc, 0x82cd, 0x82d0,
	0x82d3, 0x82d6, 0x82d9, 0x82dc,
	0x82dd, 0x82de, 0x82df, 0x82e0,
	0x82e2, 0x82e4, 0x82e6, 0x82e7,
	0x82e8, 0x82e9, 0x82ea, 0x82eb,
	0x82ed, 0x82f1, 0x814a, 0x814b
};


//---------------------------------------------------------------------------
extern ST_NV Nv;


//---------------------------------------------------------------------------
IWRAM_CODE void NvExecParse(void)
{
	if(Nv.waitCnt != 0)
	{
		Nv.waitCnt--;
		return;
	}

	if(Nv.isSeWait == TRUE)
	{
		if(SeIsEnd() == FALSE)
		{
			return;
		}

		Nv.isSeWait = FALSE;
	}


	Nv.isLoop = TRUE;

	do
	{
		NvExecParseMsg();
		NvExecParseCmd();

	} while(Nv.isLoop == TRUE);
}
//---------------------------------------------------------------------------
IWRAM_CODE void NvExecParseMsg(void)
{
	u16 idx;

	while(*Nv.pCur == 0x20 || *Nv.pCur > 0x80)
	{
		if(*Nv.pCur == 0x20)
		{
			Nv.pCur++;

			Nv.msg[Nv.msgCnt++] = 0x81;
			Nv.msg[Nv.msgCnt++] = 0x40;
		}
		else if(*Nv.pCur >= 0xe0)
		{
			Nv.msg[Nv.msgCnt++] = *Nv.pCur++;
			Nv.msg[Nv.msgCnt++] = *Nv.pCur++;
		}
		else if(*Nv.pCur >= 0xa0)
		{
			idx = NvKanaToSjisTable[*Nv.pCur++ - 0xa0];

			Nv.msg[Nv.msgCnt++] = HIBYTE(idx);
			Nv.msg[Nv.msgCnt++] = LOBYTE(idx);
		}
		else
		{
			Nv.msg[Nv.msgCnt++] = *Nv.pCur++;
			Nv.msg[Nv.msgCnt++] = *Nv.pCur++;
		}
	}

	Nv.msg[Nv.msgCnt] = '\0';
	_ASSERT(Nv.msgCnt+1 < NV_MAX_MSG_SIZE);
}
//---------------------------------------------------------------------------
IWRAM_CODE void NvExecParseCmd(void)
{
//	TRACEOUT("----\n");
//	TRACEOUT("[cmd:%x pCur:%x file:%s]\n", *Nv.pCur, Nv.pCur - Nv.pHead, Nv.pStr);

	switch(*Nv.pCur++)
	{
	// !
	case 0x21:
		NvExecParseAssign();
		break;

	// %
	case 0x25:
		NvExecParsePageCall();
		break;

	// <
	case 0x3c:
		NvExecParseFor();
		break;

	// > @
	case 0x3e:
	case 0x40:
		NvExecParseJump();
		break;

	// U
	case 0x55:
		NvExecParseU();
		break;

	// Y
	case 0x59:
		NvExecParseY();
		break;

	// 
	case 0x5c:
		NvExecParseLabelCall();
		break;

	// {
	case 0x7b:
		NvExecParseIf();
		break;

	// ~
	case 0x7e:
		NvExecParseLabelCallFar();
		break;

	default:
		SystemError("[Err] NvExecParseCmd\n");
		break;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void NvExecParseAssign(void)
{
//	TRACEOUT("[Assign]\n");

	NvCalcVar();
	NvCalc();
	_ASSERT(Nv.calcCnt == 2);


	u16 val = NvCalcPop();
	u16 num = NvCalcPop();

	NvVarSet(num, val);
}
//---------------------------------------------------------------------------
IWRAM_CODE void NvExecParsePageCall(void)
{
//	TRACEOUT("[PageCall]\n");

	u16 no = NvCalcGet();

	if(no == 0)
	{
		NvCallPopPatch();
	}
	else
	{
		NvCallPushPatch(no+1, 0);
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void NvExecParseFor(void)
{
//	TRACEOUT("[For]\n");

	// [v𔲂iȂj
	Nv.pCur += 3;
	Nv.pCur = Nv.pHead + NvCurU32();
}
//---------------------------------------------------------------------------
IWRAM_CODE void NvExecParseJump(void)
{
//	TRACEOUT("[Jump]\n");

	Nv.pCur = Nv.pHead + NvCurU32();
}
//---------------------------------------------------------------------------
IWRAM_CODE void NvExecParseU(void)
{
//	TRACEOUT("[U]\n");

	switch(*Nv.pCur++)
	{
	case 'D':
		NvCalcGet();
		return;

	case 'G':
	case 'S':
		NvCalcGet();
		NvCalcGet();
		return;
	}

	_ASSERT(0 && "NvExecParseU");
}
//---------------------------------------------------------------------------
IWRAM_CODE void NvExecParseY(void)
{
//	TRACEOUT("[Y]\n");

	
	u16 no = NvCalcGet();
	NvCalcGet();

	// dlRNDBK[gɓ肽̂ŌߑłĂ܂
	if(no == 4)
	{
		Nv.var[0] = 1;
		return;
	}

	_ASSERT(0 && "NvExecParseY");
}
//---------------------------------------------------------------------------
IWRAM_CODE void NvExecParseLabelCall(void)
{
//	TRACEOUT("[LabelCall]\n");

	u32 adr = NvCurU32();

	if(adr == 0)
	{
		NvCallPopPatch();
	}
	else
	{
		NvCallPushPatch(Nv.no, adr);
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void NvExecParseIf(void)
{
//	TRACEOUT("[If]\n");

	u16 is  = NvCalcGet();
	u32 adr = NvCurU32();

	if(is == 0)
	{
		Nv.pCur = Nv.pHead + adr;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void NvExecParseLabelCallFar(void)
{
//	TRACEOUT("[LabelCallFar]\n");

	u16 no  = NvCurU16();

	if(no == 0x0000)
	{
		NvCallPopPatch();
	}
	else if(no == 0xffff)
	{
		// ֐̖߂lϐɎ擾iȂj
		NvCalc();
		NvCalcPop();
	}
	else
	{
		u32 adr = NvCurU32();
		NvCallPushPatch(no, adr);
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void NvCalc(void)
{
	u16 in, ed;

	for(;;)
	{
		if(*Nv.pCur == 0x7f)
		{
			Nv.pCur++;
			return;
		}

//		TRACEOUT("[calc val:%x ptr:%x]\n", *Nv.pCur, Nv.pCur - Nv.pHead);

		// {Zq
		if(*Nv.pCur >= 0x74 && *Nv.pCur <= 0x7e)
		{
			in = NvCalcPop();
			ed = NvCalcPop();

			switch(*Nv.pCur)
			{
			case 0x74: NvCalcPush(ed & in);     break;
			case 0x75: NvCalcPush(ed | in);     break;
			case 0x76: NvCalcPush(ed ^ in);     break;
			case 0x77: NvCalcPush(ed * in);     break;
			case 0x78: NvCalcPush(Div(ed, in)); break;
			case 0x79: NvCalcPush(ed + in);     break;
			case 0x7a: NvCalcPush(ed - in);     break;

			case 0x7b:
				if(ed == in) NvCalcPush(1);
				else         NvCalcPush(0);
				break;

			case 0x7c:
				if(ed < in)  NvCalcPush(1);
				else         NvCalcPush(0);
				break;

			case 0x7d:
				if(ed > in)  NvCalcPush(1);
				else         NvCalcPush(0);
				break;

			case 0x7e:
				if(ed != in) NvCalcPush(1);
				else         NvCalcPush(0);
				break;
			}

			Nv.pCur++;
			continue;
		}

		// ϐ(1Byte)
		if(*Nv.pCur >= 0x80 && *Nv.pCur <= 0xbf)
		{
			u16 num = NvVarGet(*Nv.pCur - 0x80);
			NvCalcPush(num);

			Nv.pCur++;
			continue;
		}

		// ϐ(2Byte)
		if(*Nv.pCur == 0xc0 && *(Nv.pCur + 1) >= 0x40)
		{
			u16 num = NvVarGet(*(Nv.pCur + 1));
			NvCalcPush(num);

			Nv.pCur += 2;
			continue;
		}

		// 萔(1Byte)
		if(*Nv.pCur >= 0x40 && *Nv.pCur <= 0x73)
		{
			NvCalcPush(*Nv.pCur - 0x40);

			Nv.pCur++;
			continue;
		}

		// 萔(2Byte)
		if(*Nv.pCur >= 0x01 && *Nv.pCur <= 0x3f)
		{
			NvCalcPush((*Nv.pCur << 8) | *(Nv.pCur + 1));

			Nv.pCur += 2;
			continue;
		}

		// 2oCgR[h
		if(*Nv.pCur == 0x00 && *(Nv.pCur + 1) >= 0x34)
		{
			NvCalcPush((*Nv.pCur << 8) | *(Nv.pCur + 1));

			Nv.pCur += 2;
			continue;
		}
		

		_ASSERT(0 && "NvCalc Err");

	} // for
}
//---------------------------------------------------------------------------
IWRAM_CODE void NvCalcVar(void)
{
//	TRACEOUT("[var val:%x ptr:%x]\n", *Nv.pCur, Nv.pCur - Nv.pHead);

	// ϐ(1Byte)
	if(*Nv.pCur >= 0x80 && *Nv.pCur <= 0xbf)
	{
		NvCalcPush(*Nv.pCur - 0x80);

		Nv.pCur++;
		return;
	}

	// ϐ(2Byte)
	if(*Nv.pCur == 0xc0 && *(Nv.pCur + 1) >= 0x40)
	{
		NvCalcPush(*(Nv.pCur + 1));

		Nv.pCur += 2;
		return;
	}

	// 萔(2Byte)
	if(*Nv.pCur >= 0x01 && *Nv.pCur <= 0x3f)
	{
		NvCalcPush((*Nv.pCur << 8) | *(Nv.pCur + 1));

		Nv.pCur += 2;
		return;
	}

	// 萔(1Byte)
	if(*Nv.pCur >= 0x40 && *Nv.pCur <= 0x73)
	{
		NvCalcPush(*Nv.pCur - 0x40);

		Nv.pCur++;
		return;
	}

	// 2oCgR[h
	if(*Nv.pCur == 0x00 && *(Nv.pCur + 1) >= 0x34)
	{
		NvCalcPush((*Nv.pCur << 8) | *(Nv.pCur + 1));

		Nv.pCur += 2;
		return;
	}


	_ASSERT(0 && "NvCalcVar Err");
}
//---------------------------------------------------------------------------
IWRAM_CODE void NvCalcPush(u16 num)
{
//	TRACEOUT("[calc push: %x]\n", num);

	_ASSERT(Nv.calcCnt+1 < NV_MAX_CALC_CNT);
	Nv.calcTbl[Nv.calcCnt++] = num;
}
//---------------------------------------------------------------------------
IWRAM_CODE u16 NvCalcPop(void)
{
	_ASSERT(Nv.calcCnt != 0);


	u16 num = Nv.calcTbl[--Nv.calcCnt];
//	TRACEOUT("[calc pop: %x]\n", num);

	return num;
}
//---------------------------------------------------------------------------
IWRAM_CODE u16 NvCalcGet(void)
{
	NvCalc();
	_ASSERT(Nv.calcCnt == 1);


	u16 ret = NvCalcPop();
//	TRACEOUT("[calcGet ret:%x]\n", ret);

	return ret;
}
//---------------------------------------------------------------------------
IWRAM_CODE void NvVarSet(u16 num, u16 val)
{
	_ASSERT(num < NV_MAX_VAR_CNT);


//	TRACEOUT("[!var%d:%d!]\n", num, val);
	Nv.var[num] = val;
}
//---------------------------------------------------------------------------
IWRAM_CODE u16 NvVarGet(u16 num)
{
	_ASSERT(num < NV_MAX_VAR_CNT);
	return Nv.var[num];
}
//---------------------------------------------------------------------------
IWRAM_CODE void NvSelPush(void)
{
	_Strncpy((char*)Nv.selBuf[Nv.selCnt], (char*)Nv.msg, NV_MAX_SEL_BUF_SIZE);
	Nv.selNum[Nv.selCnt] = Nv.selRegCnt;

	Nv.selCnt++;
	Nv.selRegCnt++;
}
//---------------------------------------------------------------------------
IWRAM_CODE void NvCallPush(u16 no, u32 adr)
{
	_ASSERT(Nv.callCnt < NV_MAX_CALL_CNT);
//	TRACEOUT("[callpush cnt:%x no:%x adr:%x]\n", Nv.callCnt, no, adr);


	Nv.callNo[Nv.callCnt] = Nv.no;
	Nv.callAdr[Nv.callCnt] = Nv.pCur - Nv.pHead;
	Nv.callCnt++;

	NvSetSco(no, adr);
}
//---------------------------------------------------------------------------
IWRAM_CODE void NvCallPush2(u16 no, u32 adr)
{
	_ASSERT(Nv.callCnt < NV_MAX_CALL_CNT);

	Nv.callNo[Nv.callCnt] = no;
	Nv.callAdr[Nv.callCnt] = adr;
	Nv.callCnt++;
}
//---------------------------------------------------------------------------
IWRAM_CODE void NvCallPushPatch(u16 no, u32 adr)
{
	// #SYSTEM35, NVKEYIN.ADV, NVGRP.ADV
	if(no == 0 || no == 5 || no == 6)
	{
		return;
	}

	// NVSYSTEM.ADVȊO
	if(no != 3)
	{
		NvCallPush(no, adr);
		return;
	}


//	TRACEOUT("[Patch no:%x adr:%x pStr:%s]\n", no ,adr, FileGetScoStr(no));

	switch(Nv.var[22])
	{
	case 100:
		if(Nv.isTextClear1st == TRUE)
		{
			TRACEOUT("[Text Clear]\n");
			TextClear();
			Nv.isLoop = FALSE;
		}
		Nv.isTextClear1st = TRUE;
		break;

	case 101:
//		TRACEOUT("[Text Lf]\n");
		TextSetLf();
		break;

	case 102:
		TRACEOUT("[%s]\n", Nv.msg);
		TextSetBufWork((u16*)Nv.msg, Nv.msgCnt / 2, TRUE);
		Nv.msgCnt = 0;

		TextShow();

		NvSetAct(NV_ACT_KEY);
		Nv.isLoop = FALSE;
		break;

	case 150:
		TRACEOUT("[Select Reg msg:%s]\n", Nv.msg);
		NvSelPush();
		Nv.msgCnt = 0;
		break;

	case 151:
		TRACEOUT("[Select Start msg:%s]\n", Nv.msg);
		NvSelPush();
		Nv.msgCnt = 0;

		NvSetAct(NV_ACT_SELECT);
		Nv.isLoop = FALSE;
		break;

	case 154:
		TRACEOUT("[Select RegAdd + TextBufClear]\n");
		Nv.msgCnt = 0;
		Nv.selRegCnt++;
		break;

	case 200:
		TRACEOUT("[Img buf:%x file:G%03d.img]\n", Nv.var[1]-1, Nv.var[2]);

		if(Nv.var[1]-1 == 3)
		{
			ImgSetCg(0, 0);
			ImgSetCg(1, Nv.var[2]);	// TODO {̓LN^𒆉\
		}
		else if(Nv.var[1]-1 == 2)
		{
			ImgSetCg(0, Nv.var[2]);
			ImgSetCg(1, 0);
		}
		else
		{
			ImgSetCg(Nv.var[1]-1, Nv.var[2]);
		}
		ImgSetDraw();
		break;

	case 202:
		TRACEOUT("[Copy+Img Update]\n");
		ImgSetEffect2(IMG_EFFECT_NORMAL, 0);
		ImgSetDraw();

		Nv.isLoop = FALSE;
		break;

	case 203:
		TRACEOUT("[Effect no:%x wait:%x]\n", Nv.var[1], Nv.var[2]);
		if(Nv.isSkip == FALSE)
		{
			ImgSetEffect(Nv.var[1], Nv.var[2]);
		}
		else
		{
			ImgSetEffect2(IMG_EFFECT_NORMAL, 0);
		}
		ImgSetDraw();
		break;

	case 206:
		TRACEOUT("[Img Update]\n");
		ImgSetDraw();

		Nv.isLoop = FALSE;
		break;

	case 209:
		TRACEOUT("[Img Clear]\n");
		ImgSetCg(0, 0);
		ImgSetCg(1, 0);
		ImgSetDraw();
		break;

	case 302:
		TRACEOUT("[Wait cnt:%x]\n", Nv.var[1]);
		if(Nv.isSkip == FALSE)
		{
			Nv.waitCnt = Nv.var[1];
			Nv.isLoop = FALSE;
		}
		break;

	case 500:
		TRACEOUT("[Text Hide+Wait cnt:%x]\n", Nv.var[1]);
		TextHide();

		if(Nv.isSkip == FALSE)
		{
			Nv.waitCnt = Nv.var[1];
			Nv.isLoop = FALSE;
		}
		break;

	case 600:
		TRACEOUT("[Se no:%x]\n", Nv.var[1]);
		SePlay(Nv.var[1] ,FALSE);
		break;

	case 601:
		TRACEOUT("[Se2 no:%x]\n", Nv.var[1]);
		SePlay(Nv.var[1] ,FALSE);
		break;

	case 602:
		TRACEOUT("[Se3 no:%x]\n", Nv.var[1]);
		SePlay(Nv.var[1] ,FALSE);
		break;

	case 604:
		TRACEOUT("[Bgm no:%x]\n", Nv.var[1]);
		if(Nv.var[1] == 0)
		{
			BgmStop();
		}
		else
		{
			BgmPlay(Nv.var[1] + 1, TRUE);
		}
		break;

	case 605:
		TRACEOUT("[Se Wait]\n");
		Nv.isSeWait = TRUE;
		Nv.isLoop = FALSE;
		break;

	case 901:
		TRACEOUT("[SysFlag1?]\n");
		Nv.var[258] = Nv.var[1];
		Nv.var[0]   = Nv.var[259];
		break;

	case 902:
		TRACEOUT("[SysFlag2?]\n");
		Nv.var[258] = Nv.var[1];
		Nv.var[259] = Nv.var[2];
		Nv.var[206] = 1;
		break;

	case 103:	// J[\֌WH
	case 104:	// eLXgH
		// fobOO\
		break;

	case 105:
	case 106:
	case 153:
	case 201:
	case 205:
	case 301:
	case 401:
	case 402:
	case 603:
	case 912:
	case 913:
	case 914:
	case 915:
	case 920:
		TRACEOUT("[Ignore] NvCallPushPatch var[22]=%d\n", Nv.var[22]);
		break;

	default:
		TRACEOUT("[Err] NvCallPushPatch var[22]=%d\n", Nv.var[22]);
		break;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void NvCallPop(void)
{
ReStart:

	if(Nv.callCnt == 0)
	{
		TRACEOUT("[callpop chapter end]");

		Nv.isLoop = FALSE;
		NvSetAct(NV_ACT_END_READY);
		return;
	}

	Nv.callCnt--;
	u16 no  = Nv.callNo[Nv.callCnt];
	u16 adr = Nv.callAdr[Nv.callCnt];

//	TRACEOUT("[callpop: cnt:%x no:%x adr:%x]\n", Nv.callCnt, no, adr);

	if(no == 90)
	{
		AnimeSetDat(adr);
		Nv.isLoop = FALSE;

		goto ReStart;
	}

	NvSetSco(no, adr);
}
//---------------------------------------------------------------------------
IWRAM_CODE void NvCallPopPatch(void)
{
	if(Nv.callCnt == 0)
	{
		switch(Nv.no)
		{
		// cO~
		case 31:
			if(Nv.var[23] == 1)
			{
				// cO~тɂȂ
				NvCallPush2(32, 0x002a);
			}
			break;

		// ^Jq
		case 33:
			if(Nv.var[23] == 1)
			{
				// T`z[g
				NvCallPush2(36, 0x002a);
			}
			else if(Nv.var[23] == 2)
			{
				// cO~[g
				NvCallPush2(35, 0);
				NvCallPush2(34, 0x002a);
			}
			break;

		// ^Jq{T`z[g
		case 36:
			if(Nv.var[23] == 1)
			{
				// cO~I
				NvCallPush2(37, 0x002a);
			}
			else if(Nv.var[23] == 2)
			{
				// T`zI
				NvCallPush2(38, 0);
			}
			break;

		// JiR
		case 43:
			if(Nv.var[23] == 1)
			{
				// Ă
				NvCallPush2(44, 0x002a);
			}
			else if(Nv.var[23] == 2)
			{
				// avĂ
				NvCallPush2(45, 0x002a);
			}
			break;
		
		// JiR{avĂ
		case 45:
			if(Nv.var[23] == 2)
			{
				// uȂv̏ꍇAjŃ[g
				NvCallPush2(46, 0x002a);
			}
		} // switch
	} // if


	NvCallPop();
}
//---------------------------------------------------------------------------
IWRAM_CODE u16 NvCurU16(void)
{
	u16 num = *Nv.pCur++;

	num |= ((*Nv.pCur++) << 8);

	return num;
}
//---------------------------------------------------------------------------
IWRAM_CODE u32 NvCurU32(void)
{
	u32 num = *Nv.pCur++;

	num |= ((*Nv.pCur++) << 8);
	num |= ((*Nv.pCur++) << 16);
	num |= ((*Nv.pCur++) << 24);

	return num;
}
