#include "siori.h"
#include "libmy/sram.h"

#include "libmy/fade.h"
#include "bgm.h"
#include "img.h"
#include "nv.h"
#include "text.h"

#include "history.h"
#include "se.h"
#include "anime.h"
#include "menu.h"


//---------------------------------------------------------------------------
// OQ
extern ST_BGM  Bgm;
extern ST_IMG  Img;
extern ST_NV   Nv;
extern ST_TEXT Text;
extern ST_FADE Fade;


//---------------------------------------------------------------------------
ST_SIORI Siori;


//---------------------------------------------------------------------------
EWRAM_CODE void SioriInit(void)
{
	_Memset(&Siori, 0x00, sizeof(ST_SIORI));

	Siori.size = 2 + TEXT_SIORI_CX*2 + sizeof(ST_BGM) + sizeof(ST_IMG) + sizeof(ST_NV) + sizeof(ST_TEXT) + 2;

	TRACEOUT("[SioriSize: %x]\n", Siori.size);
	_ASSERT(Siori.size * SIORI_MAX_CNT < SIORI_SRAM_SIZE);
}
//---------------------------------------------------------------------------
EWRAM_CODE void SioriSave(u16 no)
{
	_ASSERT(no < SIORI_MAX_CNT);


	TRACEOUT("[Siori Save no:%d]\n", no);

	u16 i;
	u8* p;

	SramSeek(Siori.size * no);
	SramWrite8('A');
	SramWrite8('T');

	// ^Cg
	p = (u8*)TextGetBufSiori();
	for(i=0; i<TEXT_SIORI_CX*2; i++)
	{
		SramWrite8(*p++);
	}

	// BGM
	p = (u8*)&Bgm;
	for(i=0; i<sizeof(ST_BGM); i++)
	{
		SramWrite8(*p++);
	}

	// Img
	p = (u8*)&Img;
	for(i=0; i<sizeof(ST_IMG); i++)
	{
		SramWrite8(*p++);
	}

	// Nv
	p = (u8*)&Nv;
	for(i=0; i<sizeof(ST_NV); i++)
	{
		SramWrite8(*p++);
	}

	// Text
	p = (u8*)&Text;
	for(i=0; i<sizeof(ST_TEXT); i++)
	{
		SramWrite8(*p++);
	}

	// Fade
	SramWrite16(Fade.msg);
}
//---------------------------------------------------------------------------
EWRAM_CODE bool SioriLoad(u16 no)
{
	TRACEOUT("[Siori Load no:%d]\n", no);

	if(SioriIsSram(no) == FALSE)
	{
		TRACEOUT("[Failure]\n");
		return FALSE;
	}


	u16 i;
	u8* p;

	SramSeek(Siori.size * no + 2 + TEXT_SIORI_CX*2);

	// BGM
	p = (u8*)&Bgm;
	for(i=0; i<sizeof(ST_BGM); i++)
	{
		*p++ = SramRead8();
	}

	// Img
	p = (u8*)&Img;
	for(i=0; i<sizeof(ST_IMG); i++)
	{
		*p++ = SramRead8();
	}

	// Nv
	p = (u8*)&Nv;
	for(i=0; i<sizeof(ST_NV); i++)
	{
		*p++ = SramRead8();
	}

	// Text
	p = (u8*)&Text;
	for(i=0; i<sizeof(ST_TEXT); i++)
	{
		*p++ = SramRead8();
	}

	// Fade
	u16 cnt = SramRead8();
	cnt |= SramRead8() << 8;

	// A
	SeStop();
	BgmPlayRestart();

	FadeDirect(cnt);
	ImgSetEffect2(IMG_EFFECT_NORMAL, 0);
	ImgSetDraw();

	HistoryInit();

	if(Nv.act == NV_ACT_SELECT)
	{
		Nv.actStep = 1;
	}

	return TRUE;
}
//---------------------------------------------------------------------------
EWRAM_CODE char* SioriGetName(u16 no)
{
	if(SioriIsSram(no) == FALSE)
	{
		return "|||||||||||";
	}

	SramSeek(Siori.size * no + 2);
	return (char*)SramGetPointer();
}
//---------------------------------------------------------------------------
EWRAM_CODE bool SioriIsSram(u16 no)
{
	SramSeek(Siori.size * no);

	if(SramRead8() != 'A') return FALSE;
	if(SramRead8() != 'T') return FALSE;

	return TRUE;
}
