#include "text.h"
#include "libmy/spr.h"
#include "libmy/bios_arm.h"
#include "history.h"
#include "res.h"
#include "img.h"


//---------------------------------------------------------------------------
ST_TEXT Text;


//---------------------------------------------------------------------------
EWRAM_CODE void TextInit(void)
{
	_Memset(&Text, 0x00, sizeof(ST_TEXT));

	TextClearDat();
}
//---------------------------------------------------------------------------
EWRAM_CODE void TextExec(void)
{
	TextExecChr();
	TextExecCur();
}
//---------------------------------------------------------------------------
EWRAM_CODE void TextExecChr(void)
{
	if(Text.isDraw == FALSE)
	{
		return;
	}

	if(Text.waitCnt != 0)
	{
		Text.waitCnt--;
		return;
	}
	Text.waitCnt = Text.waitNum;


	if(Text.drawCnt < Text.workCnt)
	{
		TextDrawChr(Text.workBuf[Text.drawCnt++], TRUE);
		return;
	}

	Text.isDraw = FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void TextExecCur(void)
{
	if(Text.isCurDraw == FALSE)
	{
		return;
	}

	if(Text.curWaitCnt != 0)
	{
		Text.curWaitCnt--;
		return;
	}
	Text.curWaitCnt = TEXT_CURSOR_WAIT_CNT;


	if(Text.isCurBlink == TRUE)
	{
		Text.isCurBlink = FALSE;
		SprShowCursor();
	}
	else
	{
		Text.isCurBlink = TRUE;
		SprHideCursor();
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void TextShow(void)
{
	SprShowWindow();
}
//---------------------------------------------------------------------------
EWRAM_CODE void TextHide(void)
{
	SprHideWindow();
}
//---------------------------------------------------------------------------
EWRAM_CODE void TextShowCur(void)
{
	if(Text.isCurDraw == TRUE)
	{
		return;
	}

	Text.isCurDraw  = TRUE;
	Text.isCurBlink = TRUE;
	Text.curWaitCnt = 0;

	u16 x = TEXT_CURSOR_SX + Text.x * (SPR_FONT_IMG_CX);
	u16 y = TEXT_CURSOR_SY + Text.y * (SPR_FONT_IMG_CY + SPR_FONT_IMG_BLANK_CY);
	SprMoveCursor(x, y);
}
//---------------------------------------------------------------------------
EWRAM_CODE void TextHideCur(void)
{
	Text.isCurDraw = FALSE;

	SprHideCursor();
}
//---------------------------------------------------------------------------
EWRAM_CODE void TextClear(void)
{
	TextClearBuf();
	TextClearDat();
	TextClearXY();
}
//---------------------------------------------------------------------------
EWRAM_CODE void TextClearBuf(void)
{
	HistoryAddBuf(Text.outBuf);
	BiosExecFixClear(Text.outBuf, TEXT_SCREEN_CX * TEXT_SCREEN_CY * 2);
}
//---------------------------------------------------------------------------
EWRAM_CODE void TextClearDat(void)
{
	SprClearDat();
}
//---------------------------------------------------------------------------
EWRAM_CODE void TextClearXY(void)
{
	Text.x = 0;
	Text.y = 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void TextSetLf(void)
{
	if(Text.x == 0)
	{
		return;
	}

	Text.x = 0;
	Text.y++;
}
//---------------------------------------------------------------------------
IWRAM_CODE void TextSetBufWork(u16* pStr, u16 len, bool isWait)
{
	// ̍ږeLXg蒊o܂
	TextSetBufSiori(pStr, len);

	// ʂɓ肫邩`FbN܂
	if(TextCalcStrBoundary(pStr, len) == FALSE)
	{
		TextClear();
		TextCalcStrBoundary(pStr, len);
	}

	// \̓EFCgȂ`FbN܂
	if(isWait == TRUE && Text.waitNum != 0)
	{
		Text.waitCnt = 0;
		Text.drawCnt = 0;
		Text.isDraw  = TRUE;
	}
	else
	{
		TextDrawStr(Text.workBuf, Text.workCnt, TRUE);
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void TextSetBufSiori(u16* pStr, u16 len)
{
	u16* pS = pStr;
	u16  cnt;
	u16  i;

	if(len >= TEXT_SIORI_CX-1)
	{
		cnt = TEXT_SIORI_CX-1;
	}
	else
	{
		cnt = len;
	}

	for(i=0; i<cnt; i++)
	{
		Text.sioriBuf[i] = *pS++;
	}

	for(; i<TEXT_SIORI_CX; i++)
	{
		Text.sioriBuf[i] = 0x0000;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE u16* TextGetBufSiori(void)
{
	return Text.sioriBuf;
}
//---------------------------------------------------------------------------
EWRAM_CODE void TextSetWaitNum(u16 num)
{
	Text.waitNum = num;
}
//---------------------------------------------------------------------------
EWRAM_CODE u16 TextGetWaitNum(void)
{
	return Text.waitNum;
}
//---------------------------------------------------------------------------
// ֑̏
// pSrc eLXgʂ炠ӂꍇAFALSE ߂lɕԂ܂
//  TRUEBText.workBuf Text.workCnt ɒli[܂
IWRAM_CODE bool TextCalcStrBoundary(u16* pSrc, u16 len)
{
	u16 x = Text.x;
	u16 y = Text.y;
	u16 s = 0;
	u16 d = 0;
	u16 code;

	// sꍇAsɈړ܂
	if(x >= TEXT_SCREEN_CX)
	{
		x = 0;
		y++;
	}

	// 1O̍sꍇA󔒂PčsɈړ܂
	if(x == (TEXT_SCREEN_CX-1))
	{
		Text.workBuf[d++] = 0x0000;

		x = 0;
		y++;
	}

	// ŏ̕󔒂̏ꍇA󔒂܂Bȍ~̋󔒂̓XLbv܂
	if(pSrc[s] == 0x4081)
	{
		Text.workBuf[d++] = 0x0000;

		s++;
		x++;
	}

	for(;;)
	{
		_ASSERT(d <= (TEXT_SCREEN_CX * TEXT_SCREEN_CY) && "workBuf overflow");

		if(d >= TEXT_SCREEN_CX * TEXT_SCREEN_CY)
		{
			goto End;
		}
		else if(s >= len)
		{
			goto End;
		}
		else if(y >= TEXT_SCREEN_CY)
		{
			return FALSE;
		}

		code = pSrc[s++];


		if(code == 0x4081)
		{
			// 󔒃`FbN
			// EMPTY
		}
		else if(x == 0 && TextIsChrBoundaryBefore(code) == TRUE && d != 0)
		{
			// s`FbN
			Text.workBuf[d+0] = Text.workBuf[d-1];
			Text.workBuf[d-1] = 0x0000;
			Text.workBuf[d+1] = code;
			d += 2;

			y++;
			x = 2;
		}
		else if(x == (TEXT_SCREEN_CX-1) && TextIsChrBoundaryAfter(code) == TRUE)
		{
			// s`FbN
			Text.workBuf[d+0] = 0x0000;
			Text.workBuf[d+1] = code;
			d += 2;

			y++;
			x = 1;
		}
		else
		{
			// ʏ̏ꍇ
			Text.workBuf[d++] = code;
			x++;

			if(x >= TEXT_SCREEN_CX)
			{
				x = 0;
				y++;
			}
		}
	}

End:
	Text.workCnt = d;

	return TRUE;
}
//---------------------------------------------------------------------------
IWRAM_CODE bool TextIsChrBoundaryBefore(u16 code)
{
	switch(code)
	{
	case 0xE182:	// 
	case 0xE382:	// 
	case 0xE582:	// 
	case 0xC182:	// 
	case 0x4281:	// B
	case 0x4181:	// A
	case 0x4581:	// E
	case 0x6381:	// c
	case 0x7881:	// x
	case 0x7681:	// v
	case 0x6A81:	// j
	case 0x4881:	// H
	case 0x4981:	// I
	case 0x7C81:	// |
	case 0x5B81:	// [
	case 0x8383:	// 
	case 0x8583:	// 
	case 0x8783:	// 
	case 0x6283:	// b
	case 0x9683:	// 
	case 0x9F82:	// 
	case 0xA182:	// 
	case 0xA382:	// 
	case 0xA582:	// 
	case 0xA782:	// 
	case 0x4083:	// @
	case 0x4283:	// B
	case 0x4483:	// D
	case 0x4683:	// F
	case 0x4883:	// H
	case 0x4481:	// D
	case 0x6081:	// `
		return TRUE;
	}

	return FALSE;
}
//---------------------------------------------------------------------------
IWRAM_CODE bool TextIsChrBoundaryAfter(u16 code)
{
	switch(code)
	{
	case 0x7781:	// w
	case 0x7581:	// u
	case 0x6981:	// i
		return TRUE;
	}

	return FALSE;
}
//---------------------------------------------------------------------------
IWRAM_CODE void TextDrawChr(u16 code, bool isBuf)
{
	if(Text.x >= TEXT_SCREEN_CX)
	{
		Text.x = 0;
		Text.y++;
	}

	_ASSERT(Text.x < TEXT_SCREEN_CX && Text.y < TEXT_SCREEN_CY);
	SprDrawDatChr(Text.x, Text.y, code);

	if(isBuf == TRUE)
	{
		Text.outBuf[(Text.y * TEXT_SCREEN_CX) + Text.x] = code;
	}

	Text.x++;
}
//---------------------------------------------------------------------------
IWRAM_CODE void TextDrawStr(u16* pStr, u16 len, bool isBuf)
{
	u16 i;

	for(i=0; i<len; i++)
	{
		TextDrawChr(pStr[i], isBuf);
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void TextDrawStrXy(u16 x, u16 y, u16* pStr)
{
	u16 tx = Text.x;
	u16 ty = Text.y;

	Text.x = x;
	Text.y = y;

	u16 len = _Strlen((char*)pStr) / 2;
	TextDrawStr(pStr, len, FALSE);

	Text.x = tx;
	Text.y = ty;
}
//---------------------------------------------------------------------------
IWRAM_CODE void TextDrawStrXyGray(u16 x, u16 y, u16* pStr)
{
	SprSetImgGray();
	TextDrawStrXy(x, y, pStr);
	SprSetImgWhite();
}
//---------------------------------------------------------------------------
IWRAM_CODE void TextDrawFull(u16* pStr)
{
	u16 x, y;

	for(y=0; y<TEXT_SCREEN_CY; y++)
	{
		for(x=0; x<TEXT_SCREEN_CX; x++)
		{
			SprDrawDatChr(x, y, *pStr++);
		}
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void TextDrawFullGray(u16* pStr)
{
	SprSetImgGray();
	TextDrawFull(pStr);
	SprSetImgWhite();
}
//---------------------------------------------------------------------------
IWRAM_CODE void TextDrawRestore(void)
{
	TextDrawFull(Text.outBuf);
}
//---------------------------------------------------------------------------
EWRAM_CODE bool TextIsEnd(void)
{
	return (Text.isDraw == TRUE) ? FALSE : TRUE;
}
